// 
// Mar 2008, Wessel de Roode
// Impulse system from Dr. Alexander Elder in his book trading for a living, and Come into my trading room
//
// Version 1.1
//  - Now with candles sticks filled or open. Known bug after changing the chart type it does not update. 
//    This is because there was an unsupported function used to find out whenever it is a bar or a OHLC chart
//  - Changed the green bars into darkgreen for better visibility on white and black background

// You can use BarColor along with CandleOutlineColor. 
// If CandleOutlineColor is not set when you use BarColor the outline color will just take the color of BarColor.


#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Dr. Elder's ImpulseSystem. It uses the direction of an EMA and the up or down ticks from the MACD histogram
    /// </summary>
    [Description("Dr. Elder's ImpulseSystem. It uses the direction of an EMA and the up or down ticks from the MACD histogram")]
    public class ImpulseSystem : Indicator
    {
        #region Variables
        // Wizard generated variables
            private int ema = 12; // Default setting for Ema
            private int fast = 12; // Default setting for Fast
            private int slow = 26; // Default setting for Slow
            private int smooth = 9; // Default setting for Smooth
//			private bool calcOnClose = false;
        // User defined variables (add any user defined variables below)
		
			private	DataSeries		fastEma;
			private	DataSeries		slowEma;
			private	DataSeries		diffArr;
			private	DataSeries		avg;

        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.Transparent), PlotStyle.Line, "TrendEma"));
			
            //CalculateOnBarClose	= false;
            Overlay				= true;
            PriceTypeSupported	= false;
			
			fastEma	= new DataSeries(this);
			slowEma	= new DataSeries(this);
			diffArr = new DataSeries(this);
			avg		= new DataSeries(this);
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			
			if (CurrentBar == 0){
				fastEma.Set(Input[0]);
				slowEma.Set(Input[0]);
				Value.Set(0);
				avg.Set(0);	
			}
			else{
				
				fastEma.Set((2.0 / (1 + Fast)) * Input[0] + (1 - (2.0 / (1 + Fast))) * fastEma[1]);
				slowEma.Set((2.0 / (1 + Slow)) * Input[0] + (1 - (2.0 / (1 + Slow))) * slowEma[1]);
				
				double macd		= fastEma[0] - slowEma[0];
				double macdAvg	= (2.0 / (1 + Smooth)) * macd + (1 - (2.0 / (1 + Smooth))) * avg[1];
				
				diffArr.Set(macd - macdAvg);
				
				Value.Set(macd);
				avg.Set(macdAvg);
				
				double thisEma = EMA(ema)[0];				
				TrendEma.Set(thisEma);	
				
				double prevEma = EMA(ema)[1];
							
				//Log(ChartControl.ChartStyleType.ToString() ,LogLevel.Error);
				
				if( diffArr.Get(CurrentBar)>diffArr.Get(CurrentBar - 1) && 
					prevEma < thisEma ){
											
					CandleOutlineColor = Color.Green;						
					if(Open[0]<Close[0] && ChartControl.ChartStyleType == ChartStyleType.CandleStick ) {
						BarColor  = Color.Transparent;
					} else {
						BarColor  = Color.Green;
					}															
					
				} else if( diffArr.Get(CurrentBar)<diffArr.Get(CurrentBar - 1) && 
					prevEma > thisEma ){

					CandleOutlineColor = Color.Red;						
					if(Open[0]<Close[0] && ChartControl.ChartStyleType == ChartStyleType.CandleStick ) {
						BarColor  = Color.Transparent;
					} else {
						BarColor  = Color.Red;
					}															
						
				} else {

					CandleOutlineColor = Color.Blue;						
					if(Open[0]<Close[0] && ChartControl.ChartStyleType == ChartStyleType.CandleStick ) {									
						BarColor  = Color.Transparent;
					} else {
						BarColor  = Color.Blue;
					}															
					
				}				
			}
            
        }

        #region Properties
	
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries TrendEma
        {
            get { return Values[0]; }
        }

        [Description("Number of bars for the trend EMA")]
        [Category("Parameters")]
        public int Ema
        {
            get { return ema; }
            set { ema = Math.Max(1, value); }
        }

        [Description("Number of bars for the fast MACD period")]
        [Category("Parameters")]
        public int Fast
        {
            get { return fast; }
            set { fast = Math.Max(1, value); }
        }

        [Description("Number of bars for the slow MACD period")]
        [Category("Parameters")]
        public int Slow
        {
            get { return slow; }
            set { slow = Math.Max(1, value); }
        }

        [Description("Number of bars for the fast MACD smoothing")]
        [Category("Parameters")]
        public int Smooth
        {
            get { return smooth; }
            set { smooth = Math.Max(1, value); }
        }
		
//		[Description("Calculate on bar close")]
//        [Category("Parameters")]
//        public bool CalOnBarClose
//        {
//            get { return calcOnClose; }
//            set { calcOnClose = value; }
//        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private ImpulseSystem[] cacheImpulseSystem = null;

        private static ImpulseSystem checkImpulseSystem = new ImpulseSystem();

        /// <summary>
        /// Dr. Elder's ImpulseSystem. It uses the direction of an EMA and the up or down ticks from the MACD histogram
        /// </summary>
        /// <returns></returns>
        public ImpulseSystem ImpulseSystem(int ema, int fast, int slow, int smooth)
        {
            return ImpulseSystem(Input, ema, fast, slow, smooth);
        }

        /// <summary>
        /// Dr. Elder's ImpulseSystem. It uses the direction of an EMA and the up or down ticks from the MACD histogram
        /// </summary>
        /// <returns></returns>
        public ImpulseSystem ImpulseSystem(Data.IDataSeries input, int ema, int fast, int slow, int smooth)
        {
            checkImpulseSystem.Ema = ema;
            ema = checkImpulseSystem.Ema;
            checkImpulseSystem.Fast = fast;
            fast = checkImpulseSystem.Fast;
            checkImpulseSystem.Slow = slow;
            slow = checkImpulseSystem.Slow;
            checkImpulseSystem.Smooth = smooth;
            smooth = checkImpulseSystem.Smooth;

            if (cacheImpulseSystem != null)
                for (int idx = 0; idx < cacheImpulseSystem.Length; idx++)
                    if (cacheImpulseSystem[idx].Ema == ema && cacheImpulseSystem[idx].Fast == fast && cacheImpulseSystem[idx].Slow == slow && cacheImpulseSystem[idx].Smooth == smooth && cacheImpulseSystem[idx].EqualsInput(input))
                        return cacheImpulseSystem[idx];

            ImpulseSystem indicator = new ImpulseSystem();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Ema = ema;
            indicator.Fast = fast;
            indicator.Slow = slow;
            indicator.Smooth = smooth;
            indicator.SetUp();

            ImpulseSystem[] tmp = new ImpulseSystem[cacheImpulseSystem == null ? 1 : cacheImpulseSystem.Length + 1];
            if (cacheImpulseSystem != null)
                cacheImpulseSystem.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheImpulseSystem = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Dr. Elder's ImpulseSystem. It uses the direction of an EMA and the up or down ticks from the MACD histogram
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.ImpulseSystem ImpulseSystem(int ema, int fast, int slow, int smooth)
        {
            return _indicator.ImpulseSystem(Input, ema, fast, slow, smooth);
        }

        /// <summary>
        /// Dr. Elder's ImpulseSystem. It uses the direction of an EMA and the up or down ticks from the MACD histogram
        /// </summary>
        /// <returns></returns>
        public Indicator.ImpulseSystem ImpulseSystem(Data.IDataSeries input, int ema, int fast, int slow, int smooth)
        {
            return _indicator.ImpulseSystem(input, ema, fast, slow, smooth);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Dr. Elder's ImpulseSystem. It uses the direction of an EMA and the up or down ticks from the MACD histogram
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.ImpulseSystem ImpulseSystem(int ema, int fast, int slow, int smooth)
        {
            return _indicator.ImpulseSystem(Input, ema, fast, slow, smooth);
        }

        /// <summary>
        /// Dr. Elder's ImpulseSystem. It uses the direction of an EMA and the up or down ticks from the MACD histogram
        /// </summary>
        /// <returns></returns>
        public Indicator.ImpulseSystem ImpulseSystem(Data.IDataSeries input, int ema, int fast, int slow, int smooth)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.ImpulseSystem(input, ema, fast, slow, smooth);
        }

    }
}
#endregion
